# http://djangosnippets.org/snippets/1031/

from django.db import models
from django.contrib.contenttypes.models import ContentType

class PolyModel(models.Model):
    """
    Model class aware of its child models, allowing for child class objects to be resolved from parent objects.

    For example:
    Contact is a parent class inheriting from PolyModel. Subclasses might be Company, Person, Artist, Label etc. 
    Basic address, email etc. fields can be added to the parent class and all subclasses will have those.
    
    Having searched your database for Contact objects (undifferentiated by class) you then want to reload the 
    chosen object as the subclass that it really is:

    thing.as_leaf_class()
    """
    content_type = models.ForeignKey(
        ContentType, 
        editable=False, 
        null=True
    )
    class_name = models.CharField(
        max_length=32, 
        editable=False, 
        null=True
    )
    
    class Meta:
        abstract = True

    def as_leaf_class(self):
        """
        Returns the leaf class no matter where the calling instance is in the inheritance hierarchy.
        """
        try:
            return self.__getattribute__(self.class_name.lower())
        except AttributeError:
            content_type = self.content_type
            model = content_type.model_class()
            if(PolyModel in model.__bases__):
                return self
            return model.objects.get(id=self.id)
        
    def save(self, *args, **kwargs):
        """
        Save field required for leaf class resolution.
        """
        # set leaf class content type
        if not self.content_type:
            self.content_type = ContentType.objects.get_for_model(self.__class__)
        
        # set leaf class class name
        if not self.class_name:
            self.class_name = self.__class__.__name__

        super(PolyModel, self).save(*args, **kwargs)



import unittest

from django.contrib.contenttypes.models import ContentType

from snippetscream import PolyModel

class PolyTrunkModel(PolyModel):
    pass
class PolyBranchModel(PolyTrunkModel):
    pass
class PolyLeafModel(PolyBranchModel):
    pass

class PolyModel(unittest.TestCase):
    def test_save(self):
        # Leaf class content type should be set on save.
        obj = PolyLeafModel()
        obj.save()
        self.failUnless(obj.content_type == ContentType.objects.get_for_model(PolyLeafModel))
        
        # Leaf class class name should be set on save.
        self.failUnless(obj.class_name == PolyLeafModel.__name__)

        # Correct leaf class content type should be retained over base class' content type.
        base = obj.polytrunkmodel_ptr
        base.save()
        self.failUnless(base.content_type == ContentType.objects.get_for_model(PolyLeafModel))
       
        # Correct leaf class class name should be retained over base class' class name.
        self.failUnless(base.class_name == PolyLeafModel.__name__)

    def test_as_leaf_class(self):
        obj = PolyLeafModel()
        obj.save()

        # Always return the leaf class, no matter where we are in the hierarchy.
        self.failUnless(PolyTrunkModel.objects.get(id=obj.id).as_leaf_class() == obj)
        self.failUnless(PolyBranchModel.objects.get(id=obj.id).as_leaf_class() == obj)
        self.failUnless(PolyLeafModel.objects.get(id=obj.id).as_leaf_class() == obj)

        # Trunk only object should traverse to itself.
        trunk_obj=PolyTrunkModel()
        trunk_obj.save()
        self.failUnless(PolyTrunkModel.objects.get(id=trunk_obj.id).as_leaf_class() == trunk_obj)



# http://djangosnippets.org/snippets/1378/

from django.core.urlresolvers import RegexURLResolver, RegexURLPattern, Resolver404, get_resolver

__all__ = ('resolve_to_name',)

def _pattern_resolve_to_name(self, path):
    match = self.regex.search(path)
    if match:
        name = ""
        if self.name:
            name = self.name
        elif hasattr(self, '_callback_str'):
            name = self._callback_str
        else:
            name = "%s.%s" % (self.callback.__module__, self.callback.func_name)
        return name

def _resolver_resolve_to_name(self, path):
    tried = []
    match = self.regex.search(path)
    if match:
        new_path = path[match.end():]
        for pattern in self.url_patterns:
            try:
                name = pattern.resolve_to_name(new_path)
            except Resolver404, e:
                tried.extend([(pattern.regex.pattern + '   ' + t) for t in e.args[0]['tried']])
            else:
                if name:
                    return name
                tried.append(pattern.regex.pattern)
        raise Resolver404, {'tried': tried, 'path': new_path}


# here goes monkeypatching
RegexURLPattern.resolve_to_name = _pattern_resolve_to_name
RegexURLResolver.resolve_to_name = _resolver_resolve_to_name


def resolve_to_name(path, urlconf=None):
    return get_resolver(urlconf).resolve_to_name(path)



# http://djangosnippets.org/snippets/1875/

from django.conf import settings
from django.contrib.auth import models as auth_models
from django.contrib.auth.management import create_superuser
from django.db.models import signals

def create_default_superuser(app, created_models, verbosity, **kwargs):
    """
    Creates our default superuser.
    """
    try:
        auth_models.User.objects.get(username='admin')
    except auth_models.User.DoesNotExist:
        print 'Creating default superuser:\nUsername: admin\nPassword: admin\nEmail: invalid@ddress.com'
        assert auth_models.User.objects.create_superuser('admin', 'invalid@ddress.com', 'admin')
    else:
        print 'Default superuser already exists.'

if getattr(settings, 'CREATE_DEFAULT_SUPERUSER', False):
    # From http://stackoverflow.com/questions/1466827/:
    # Prevent interactive question about wanting a superuser created.  (This code
    # has to go in this otherwise empty "models" module so that it gets processed by
    # the "syncdb" command during database creation.)
    signals.post_syncdb.disconnect(
        create_superuser,
        sender=auth_models,
        dispatch_uid='django.contrib.auth.management.create_superuser'
    )

    # Trigger default superuser creation.
    signals.post_syncdb.connect(
        create_default_superuser, 
        sender=auth_models, 
        dispatch_uid='common.models.create_testuser'
    )



from setuptools import setup, find_packages

setup(
    name='django-snippetscream',
    version='0.0.3',
    description='Django app packaging the best snippets found on http://djangosnippets.org',
    long_description = open('README.rst', 'r').read() + open('AUTHORS.rst', 'r').read() + open('CHANGELOG.rst', 'r').read(),
    author='Shaun Sephton',
    author_email='shaunsephton@gmail.com',
    url='http://github.com/shaunsephton/django-snippetscream',
    packages = find_packages(),
    include_package_data=True,
    classifiers = [
        "Programming Language :: Python",
        "License :: OSI Approved :: BSD License",
        "Development Status :: 4 - Beta",
        "Operating System :: OS Independent",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
    ],
    zip_safe=False,
)
